#if !defined( OSD_SCSI3_H )
#define OSD_SCSI3_H


/* 
 * Definitions shared by FMU and HMU 
 */
/* Won't trigger/receive interrupt for status/request tail pointer. */
#define NO_TAIL_INTERRUPT

#define CONSTRAIN_SDRAM_BASE_ADDRESS
#define SUPPORT_IMAGE_FLASH
//#define SUPPORT_DEBUG_FRAME

#ifdef SUPPORT_IMAGE_FLASH
	/* If DMA_IMAGE_FLASH, image is DMAed by the firmware. 
	* Otherwise it's pushed by windows driver. */
	#define DMA_FLASH_IMAGE

	#ifndef DMA_FLASH_IMAGE
		#error "We only support DMA_FLASH_IMAGE now. There is some work to do if disable DMA_FLASH_IMAGE."
	#endif
#endif

#ifdef CONSTRAIN_SDRAM_BASE_ADDRESS
#undef SUPPORT_DEBUG_FRAME
#endif

#define HMU_PARTNER_TYPE_BIOS			0
#define HMU_PARTNER_TYPE_WINDOWS		1
#define HMU_PARTNER_TYPE_LINUX			2
#define HMU_PARTNER_TYPE_DOS			3

/*
 * SCSI Task Attribute
 */
#define TASK_ATTRIBUTE_SIMPLE		0x0
#define TASK_ATTRIBUTE_ORDERED		0x2
#define TASK_ATTRIBUTE_HEADOFQUEUE	0x1
#define TASK_ATTRIBUTE_ACA			0x4

/* 
 * SCSI Status 
 */
#define ssGOOD					0x00			/* no error */
#define ssCHECK_CONDITION		0x02			/* check sense data for status */
#define ssCONDITION_MET			0x04			/* should not happen */
#define ssBUSY					0x08			/* RAID Engine is busy, retry after some
												 * time */
#define ssINTERMEDIATE_GOOD		0x10			/* should not happen */
#define ssINTERMEDIATE_MET		0x14			/* should not happen */
#define ssRESERVATION_CONFLICT	0x18			/* should not happen */
#define ssCOMMAND_TERMINATED	0x22			/* should not happen */
#define ssQUEUE_FULL			0x28			/* RAID Engine is busy, retry after some time */

#define ssACA_ACTIVE			0x30
#define ssTASK_ABORTED			0x40

/* 
 * scsi3 opcode and CDB bits 
 */
#define svucSCSI_COMMAND		0x60

#define scsiINQUIRY				0x12
#define scsiMODE_SELECT_6		0x15
#define scsiMODE_SELECT_10		0x55
#define scsiMODE_SENSE_6		0x1A
#define scsiMODE_SENSE_10		0x5A
#define scsiREAD_6				0x08
#define scsiREAD_10				0x28
#define scsiREAD_CAPACITY		0x25
#define scsiREQUEST_SENSE		0x03
#define scsiREZERO_UNIT			0x01
#define scsiSEEK_6				0x0B
#define scsiSEEK_10				0x2B
#define scsiSTART_STOP_UNIT		0x1B
#define scsiTEST_UNIT_READY		0x00
#define scsiVERIFY				0x2F
#define scsiWRITE_6				0x0A
#define scsiWRITE_10			0x2A
#define scsiWRITE_VERIFY		0x2E
#define scsiSYNCHRONIZE_CACHE	0x35
#define scsiREAD_16				0x88			/* to be supported */
#define scsiWRITE_16			0x8A			/* to be supported */


/*
 * Message Unit Definition between HMU and FMU
 */

#define MU_CDB_LENGTH	16

#define REQUEST_PAYLOAD_VARIABLE_DATA_LENGTH	1024

#define STATUS_PAYLOAD_VARIABLE_DATA_LENGTH		128
//#define REQUEST_PAYLOAD_VARIABLE_DATA_LENGTH	1024
//#define STATUS_PAYLOAD_VARIABLE_DATA_LENGTH	20

typedef struct _MU_REQUEST_PAYLOAD
{
	U16 TaskTag;
	U8 LUN;
	U8 Target;
	U8 TaskAttribute;
	U8 TaskManagement;
	U8 PRDEntryCount;
	U8 PayloadSize;	/* Payload Size in DWORD */
	U8 CDB[MU_CDB_LENGTH];
	U8 VariableData[REQUEST_PAYLOAD_VARIABLE_DATA_LENGTH];
}MU_REQUEST_PAYLOAD, buf_space * bpMU_REQUEST_PAYLOAD;

typedef struct _MU_STATUS_PAYLOAD
{
	U16 TaskTag;
	U8 LUN;
	U8 Target;
	U8 SrbStatus;
	U8 ScsiStatus;
	U8 Reserved2;
	U8 PayloadSize;	/* Payload Size in DWORD */
	U8 VariableData[STATUS_PAYLOAD_VARIABLE_DATA_LENGTH];
}MU_STATUS_PAYLOAD, buf_space * bpMU_STATUS_PAYLOAD;

#define STATUS_PAYLOAD_HEAD_SIZE_IN_DWORD	(OFFSET_OF(MU_STATUS_PAYLOAD, VariableData)/sizeof(U32))

//TBD: We'd better use a checksum for both frame.
typedef struct _HMU_HANDSHAKE_FRAME
{
	U32 RequestBufferPhyAddr;		/* Request Payload Queue Physical Address */
	U32 RequestBufferPhyAddrHigh;
	U16 RequestPayloadSize;			/* Size of each Request Payload */
	U16	RequestPayloadCount;		/* This Request Queue buffer can hold how many Request Payload */
	U16 StatusPayloadSize;			/* Size of each Status Payload */
	U16 StatusPayloadCount;			/* This Status Queue buffer can hold how many Status Payload */

	U64	HostTime;					/* Host Time: seconds from Jan 1, 1970 (GMT). */
	U8	PartnerType;				/* Who sends this frame. */
	U8	Reserved0[7];
	U32 PartnerVersion_Major;		/* Partner Version. */
	U32 PartnerVersion_Minor;
	U32 PartnerVersion_OEM;
	U32 PartnerVersion_Build;
	U32	Reserved1[4];
} HMU_HANDSHAKE_FRAME, buf_space * bpHMU_HANDSHAKE_FRAME;

typedef struct _FMU_HANDSHAKE_FRAME
{
	U32 RequestWindowBase;
	U32 RequestEntrySize;
} FMU_HANDSHAKE_FRAME, buf_space * bpFMU_HANDSHAKE_FRAME;

#define MU_HANDSHAKE_SIGNATURE			0x55aaaa55
#define MU_MAX_REQUEST					32

/* Definition of the Doorbell Register */
#define MU_INBOUND_DOORBELL_HANDSHAKE			bit(0)
#define MU_INBOUND_DOORBELL_REQUESTHEADCHANGED	bit(1)
#define MU_INBOUND_DOORBELL_STATUSTAILCHANGED	bit(2)
#define MU_INBOUND_DOORBELL_HMUSTOPPED			bit(3)
#define MU_INBOUND_DOORBELL_RESET				bit(4)

#define MU_OUTBOUND_DOORBELL_HANDSHAKE			bit(0)
#define MU_OUTBOUND_DOORBELL_REQUESTTAILCHANGED	bit(1)
#define MU_OUTBOUND_DOORBELL_STATUSHEADCHANGED	bit(2)
#define MU_OUTBOUND_DOORBELL_BUSCHANGE			bit(3)
#define MU_OUTBOUND_DOORBELL_HASEVENT			bit(4)

#define MU_OUTBOUND_DOORBELL_FAKESTATUS			bit(31)	/* For Debugging Purpose Only */

#if defined(SUPPORT_IMAGE_FLASH) && !defined(DMA_FLASH_IMAGE)
typedef struct _APIFRAME_FLASH_SETUP
{
	U32 FirmwareImageAddress;
	U32 FirmwareImageSize;
	U32 CurrentPageNumber;	/* Used by Windows Driver Only */
	U32 SerialNumber;		/* Used by Windows Driver Only */
	U32 Reserved[2];
}APIFRAME_FLASH_SETUP, *PAPIFRAME_FLASH_SETUP;
#endif

#endif /* OSD_SCSI3_H */
