#if !defined( GEN_DATA_STRUCTURES_H )
#define GEN_DATA_STRUCTURES_H

#include "gen_typs.h"

/*
 -----------------------------------------------------------------------------------------
    void memset( OUT void* pmem, IN U8 b, IN U32 cnt );
 -----------------------------------------------------------------------------------------
 */
struct _Bitset;
typedef struct _Bitset Bitset, buf_space * bpBitset;

struct tagGenPoolItem;
typedef struct tagGenPoolItem GenPoolItem, buf_space * bpGenPoolItem;

struct tagGenPool;
typedef struct tagGenPool GenPool, buf_space * bpGenPool;

struct _GEN_LIST_ENTRY;
typedef struct _GEN_LIST_ENTRY GEN_LIST_ENTRY, *bpGEN_LIST_ENTRY;

struct _GEN_LIST_HEADER;
typedef struct _GEN_LIST_HEADER GEN_LIST_HEADER, *bpGEN_LIST_HEADER;

/* bitset definition */
#define BITSET_MAX_BITS 1024
#define BITS_PER_UNIT	( 8 * sizeof(U32) )

/*
 * Note here by default Bitset can just support up to 1024 bit. If you want to
 * support more bit, you have to define your Bitset by using BITSET_SIZE.
 */
struct _Bitset
{
	U32 sizeOfBitset;	/* bit count of the bitset */
	U32	countOfSet;
#if VDEBUG
	U32 maxCountOfSet;
#endif
	U32 bitmap[BITSET_MAX_BITS / BITS_PER_UNIT];
};

/*
 * Use BITSET_SIZE to calculate actual size if you don't want to use defult bitset
 * size (BITSET_MAX_BITS)
 */
#define BITSET_SIZE(bitCount)	(								\
	  OFFSET_OF(Bitset,bitmap)									\
	+ sizeof(U32) * ((bitCount+BITS_PER_UNIT-1)/BITS_PER_UNIT)	\
)

/* void GenerateByteSumTable(); */
void	BitsetInitialize( bpBitset bs, U32 bitCount );
void	BitsetSetBits( bpBitset bs, U32 startingIndex, U32 count );
void	BitsetClearBits( bpBitset bs, U32 startingIndex, U32 count );
U32		BitsetNumberOfSetBits( bpBitset bs );
U32		BitsetNumberOfClearBits( bpBitset bs );
void	BitsetSetBit( bpBitset bs, U32 position );
void	BitsetClearBit( bpBitset bs, U32 position );

/*
 =========================================================================================
    U32 BitsetCheckBit(bpBitset bs, U32 bitPosition);
 =========================================================================================
 */
#define BitsetCheckBit( bs, bitPosition )						 \
		(														 \
			(													 \
				((bs)->bitmap[(bitPosition) / BITS_PER_UNIT]) >> \
					((bitPosition) % BITS_PER_UNIT)				 \
			) & 0x1												 \
		)

/*
 -----------------------------------------------------------------------------------------
    pool definition
 -----------------------------------------------------------------------------------------
 */
struct tagGenPoolItem
{
	VOLATILE bpGenPoolItem	pNext;
};

struct	tagGenPool
{
	VOLATILE bpGenPoolItem	pFree;
	VOLATILE U32			elementCount_;
	VOLATILE U32			allocated_;
	PTR_SIZE				pBufferStart_;
#if VDEBUG
	VOLATILE U32			maxAllocated_;
	const char*				name_;
	U32						bufferSize_;
#endif
};

void	GenPool_Initialize(
			bpGenPool	pPool,
			bpVOID		pBuffer,
			U32			bufferSize,
			U32			entrySize );

bpVOID	GenPool_Allocate( bpGenPool pPool );

void	GenPool_Release( bpGenPool pPool, bpVOID pEntry );

void	GenPool_Dump( bpGenPool pPool, const char* title );

#define	GenPool_Avail( pPool )	((pPool)->elementCount_ - (pPool)->allocated_)

#if VDEBUG
#define	GenPool_SetName( pPool, name )	((pPool)->name_ = name)
#else
#define	GenPool_SetName( pPool, name )	
#endif


/*
 -----------------------------------------------------------------------------------------
    doubly linked list definition
 -----------------------------------------------------------------------------------------
 */
struct _GEN_LIST_ENTRY
{
	VOLATILE bpGEN_LIST_ENTRY	Flink;
	VOLATILE bpGEN_LIST_ENTRY	Blink;
};

struct _GEN_LIST_HEADER
{
	VOLATILE bpGEN_LIST_ENTRY	Flink;
	VOLATILE bpGEN_LIST_ENTRY	Blink;
	VOLATILE U32				NextPointerOffsetInListItem;
#if 1
	VOLATILE U32				ItemCount;
#endif
};

#define EntryFromItem( ListHead, Item ) \
		( (bpGEN_LIST_ENTRY) (((PU8) (Item)) + (ListHead)->NextPointerOffsetInListItem) )
#define ItemFromEntry( ListHead, Entry ) \
		( (bpVOID) (((PU8) (Entry)) - (ListHead)->NextPointerOffsetInListItem) )

void
GenList_Initialize(
	bpGEN_LIST_HEADER	ListHead,
	U32					NextOffset );

#define GenList_IsEmpty( ListHead ) ( (ListHead)->Flink == (bpGEN_LIST_ENTRY) (ListHead) )

void
GenList_AddHead(
	bpGEN_LIST_HEADER	ListHead,
	bpVOID				ListItem );

void
GenList_AddTail(
	bpGEN_LIST_HEADER	ListHead,
	bpVOID				ListItem );

void
GenList_RemoveEntry(
	bpGEN_LIST_HEADER	ListHead,
	bpVOID				ListItem );

bpVOID
GenList_RemoveHead(
	bpGEN_LIST_HEADER	ListHead );


bpVOID
GenList_RemoveTail(
	bpGEN_LIST_HEADER	ListHead );

/*
void
GenList_Move(
	bpGEN_LIST_HEADER	pSourceListHead,
	bpGEN_LIST_HEADER	pTargetListHead );
*/
/*
 =========================================================================================
    Add for CAM usage
 =========================================================================================
 */

void
GenList_Copy(
	bpGEN_LIST_HEADER	pSourceListHead,
	bpGEN_LIST_HEADER	pTargetListHead );

bpVOID
GenList_GetHead(
	bpGEN_LIST_HEADER	ListHead );

bpVOID
GenList_GetTail(
	bpGEN_LIST_HEADER	ListHead );

bpVOID
GenList_GetNext(
	bpGEN_LIST_HEADER	ListHead,
	bpVOID				ListItem );

bpVOID
GenList_GetPrev(
	bpGEN_LIST_HEADER	ListHead,
	bpVOID				ListItem );

void
GenList_InsertEntry(
	bpGEN_LIST_HEADER	ListHead,
	bpVOID				ListItem,
	bpVOID				ListItemInsert );

void
GenList_InsertBeforeEntry(
	bpGEN_LIST_HEADER	ListHead,
	bpVOID				ListItem,
	bpVOID				ListItemInsert );

/*
 -----------------------------------------------------------------------------------------
    Link list in DIMM definition
 -----------------------------------------------------------------------------------------
 */
typedef struct	_DIMM_SLIST_HEADER
{
	bpVOID	pCAM;						/* needs CAM to access DIMM */
	U32		ListHead;
	U32		ListTail;
	U32		ListItemSize;				/* item size is provided for data copy to/from
										 * DIMM */
	U32		NextPointerOffsetInListItem;
}
DIMM_SLIST_HEADER, *bpDIMM_SLIST_HEADER;

void DimmSList_Initialize(
		DIMM_SLIST_HEADER*	DimmListHead,
		bpVOID				pCAM,
		U32					ListItemSize,
		U32					NextPointerOffsetInListItem );

void DimmSList_AddHead(
		DIMM_SLIST_HEADER*	DimmListHead,
		const void*			ListItem,	/* list item will be copied into DIMM if not NULL */
		U32					DimmAddress );

void DimmSList_AddTail(
		DIMM_SLIST_HEADER*	DimmListHead,
		const void*			ListItem,	/* list item will be copied into DIMM if not NULL */
		U32					DimmAddress );

void DimmSList_RemoveHead( DIMM_SLIST_HEADER* DimmListHead );

BoOL DimmSList_IsEmpty( DIMM_SLIST_HEADER* DimmListHead );

void DimmSList_Clear( DIMM_SLIST_HEADER* DimmListHead );

typedef struct _DIMM_PACKET_LIST
{
	GEN_LIST_HEADER		PacketHeaderList;
	DIMM_SLIST_HEADER	DimmPacketList;
}
DIMM_PACKET_LIST, *PDIMM_PACKET_LIST;
/*
void
DimmPktList_Move(
	PDIMM_PACKET_LIST pSourceList,
	PDIMM_PACKET_LIST pTargetList );
*/

/*
 -----------------------------------------------------------------------------------------
    DIMM pool definition
 -----------------------------------------------------------------------------------------
 */
typedef struct _DIMM_POOL
{
	U32		DimmAddress;
	U32		ElementSize;
	U32		NextPosition;
	Bitset	DimmBitMap;
} DIMM_POOL;

#define DIMM_POOL_SIZE(bitCount)	( OFFSET_OF(DIMM_POOL,DimmBitMap) + BITSET_SIZE(bitCount) )

void	DimmPool_Initialize(
			DIMM_POOL*	pPool,
			U32			ElementSize,
			U32			DimmSize,
			U32			DimmAddress );

void DimmPool_InitializeEx(
			DIMM_POOL* pPool,
			U32 ElementSize,
			U32 DimmSize,
			U32 DimmAddress,
			U32 MaximumBitCount );

U32		DimmPool_Allocate( DIMM_POOL* pPool );

void	DimmPool_Release( DIMM_POOL* pPool, U32 DimmAddress );

BoOL	DimmPool_AddressIncluded( DIMM_POOL* pPool, U32 DimmAddress );

/*
 -----------------------------------------------------------------------------------------
    DIMM stack definition
 -----------------------------------------------------------------------------------------
 */
struct _DimmStack_Pool;
typedef struct _DimmStack_Pool DimmStack_Pool, buf_space * bpDimmStack_Pool;

struct _DimmStack_Pool
{
	I32			StackPointer;
	U32			StackDepth;
	DIMMADDR	DimmStack[1];
};

#define DIMMSTACKPOOL_SIZE( itemCount ) \
		( OFFSET_OF(DimmStack_Pool, DimmStack) + itemCount * sizeof(DIMMADDR) )

void		DimmStack_Initialize(
				bpDimmStack_Pool	This,
				DIMMADDR			StartOffset,
				U32					ElementCount,
				U32					ElementSize );

DIMMADDR	DimmStack_Allocate( bpDimmStack_Pool This );

void		DimmStack_Release( bpDimmStack_Pool This, DIMMADDR Offset );
#endif /* GEN_DATA_STRUCTURES_H */
