#if !defined(GEN_DATA_TYPES_H)
#define GEN_DATA_TYPES_H


/** */

/** 
predefined macros:

 Platform definitions:

  Only one (and at least one) of following macros can be defined at the same time.

	_X8616B	
	Specify 16bit x86 platform, this is used for BIOS and DOS utility

	_X8632B
	Specify 32bit x86 platform, this is used for most OS drivers.

	_IA64B
	Specify 64bit IA64 platform, this is used for IA64 OS drivers.

	_AMD64B
	Specify 64bit AMD64 platform, this is used for AMD64 OS drivers.

	_IAEFI
	Specify EFI driver for IA64 (and IA32, if avaible). This is used for EFI BIOS for IA32/64.

 Abstraction definition
	_16BPLATFORM	for all 16 bit platforms, actually we don't support any 16 bit platforms 
					other than 16bit x86 platform now.
	_32BPLATFORM	for all 32 bit platforms, actually we don't support any 32 bit platforms 
					other than 32bit x86 platform now.
	_64BPLATFORM	for all 64 bit platforms, so far we need to support both AMD64 and IA64.

 OS definitions for OS drivers:

  Only one (and at least one) of following macros can be defined at the same time.

	_OS_WINDOWS
	_OS_LINUX
	_OS_FREEBSD

 Add compiler definition here if necessary.
	_COMPILER_GCC_COMPATIBLE
	_COMPILER_MS_C_COMPATIBLE
*/

/********************************************************************************/
/* basic data type definitions */
typedef unsigned char U8;
typedef   signed char I8;

typedef unsigned short U16;
typedef   signed short I16;

#if defined(_OS_LINUX)
	#include <linux/types.h>
	/** unsigned/signed long is 64bit for AMD64, so use unsigned int instead */
	typedef unsigned int U32;
	//typedef u32 U32;
	typedef   signed int I32;
	//typedef s32 I32;
#else
	/** unsigned/signed long is 32bit for x86, IA64 and AMD64 */
	typedef unsigned long U32;
	typedef   signed long I32;
#endif

#if defined(_OS_WINDOWS)

	typedef unsigned __int64 _U64;
	typedef   signed __int64 _I64;
	#define DT64BSUPPORTED

#elif defined(_OS_LINUX)
	typedef unsigned long long _U64;
	//typedef u64 _U64;
	typedef   signed long long _I64;
	//typedef s64 _I64;
	#define DT64BSUPPORTED
	#define FAR

	#if defined(_X8632B)
	/*
 	* do_div() is NOT a C function. It wants to return
	* two values (the quotient and the remainder), but
	* since that doesn't work very well in C, what it
 	* does is:
 	*
 	* - modifies the 64-bit dividend _in_place_
	* - returns the 32-bit remainder
 	*
 	* This ends up being the most efficient "calling
 	* convention" on x86.
 	*/
	#define do_div(n,base) ({ \
		unsigned long __upper, __low, __high, __mod, __base; \
		__base = (base); \
		asm("":"=a" (__low), "=d" (__high):"A" (n)); \
		__upper = __high; \
		if (__high) { \
			__upper = __high % (__base); \
			__high = __high / (__base); \
		} \
		asm("divl %2":"=a" (__low), "=d" (__mod):"rm" (__base), "0" (__low), "1" (__upper)); \
		asm("":"=A" (n):"a" (__low),"d" (__high)); \
		__mod; \
	})
	#endif
#elif defined(_SHASTAFW)

	typedef unsigned long long _U64;
	typedef   signed long long _I64;
	#define DT64BSUPPORTED

#elif defined(_OS_FREEBSD)

	/** define _U64/_I64/DT64BSUPPORTED if your compiler support 64b integer data type */

#else

	/** define _U64/_I64/DT64BSUPPORTED if your compiler support 64b integer data type */

#endif

typedef union {
	struct {
		U32 Low;
		U32 High;
	} u;
#if defined(DT64BSUPPORTED) || defined(_SHASTAFW)
	_U64 Value;
#endif
} U64;

typedef union {
	struct {
		U32 Low;
		I32 High;
	} u;
#if defined(DT64BSUPPORTED) || defined(_SHASTAFW)
	_I64 Value;
#endif
} I64; /* we should never use I64 though */

/* GEN_FAR is needed by BIOS only */
#if defined(_X8616B)

	#define GEN_FAR  __far		/* GEN_FAR/GEN_NEAR is meaningful for BIOS/DOS APP only */
	#define GEN_NEAR __near

	#define app_space GEN_FAR
	#define buf_space GEN_NEAR
	#define dat_space GEN_NEAR
	#define str_space GEN_FAR

#else
	#define GEN_FAR 
	#define GEN_NEAR
	#define app_space
	#define buf_space
	#define dat_space
	#define str_space

#endif

typedef U8 GEN_FAR* LPU8, GEN_NEAR* PU8, buf_space* bpU8, app_space* apU8, str_space* spU8, dat_space* dpU8;
typedef I8 GEN_FAR* LPI8, GEN_NEAR* PI8, buf_space* bpI8, app_space* apI8, str_space* spI8, dat_space* dpI8;

typedef U16 GEN_FAR* LPU16, GEN_NEAR* PU16, buf_space* bpU16, app_space* apU16, str_space* spU16, dat_space* dpU16;
typedef I16 GEN_FAR* LPI16, GEN_NEAR* PI16, buf_space* bpI16, app_space* apI16, str_space* spI16, dat_space* dpI16;

typedef U32 GEN_FAR* LPU32, GEN_NEAR* PU32, buf_space* bpU32, app_space* apU32, str_space* spU32, dat_space* dpU32;
typedef I32 GEN_FAR* LPI32, GEN_NEAR* PI32, buf_space* bpI32, app_space* apI32, str_space* spI32, dat_space* dpI32;

typedef U64 GEN_FAR* LPU64, GEN_NEAR* PU64, buf_space* bpU64, app_space* apU64, str_space* spU64, dat_space* dpU64;
typedef I64 GEN_FAR* LPI64, GEN_NEAR* PI64, buf_space* bpI64, app_space* apI64, str_space* spI64, dat_space* dpI64;


typedef void GEN_FAR* LPVOID, buf_space* bpVOID, str_space* spVOID, dat_space* dpVOID;

/** 
	if you want to defined pointer determined by compiler mode, just define as DataType instead of
	pre-defined types above. 
 */

#if !defined(NULL)
#define NULL 0
#endif 

/** PTR_SIZE is necessary to convert a pointer to/from integer safely, please always use PTR_SIZE
    to define an integer if you want to save a pointer to this integer. This is important for 32/64
	bit compatibility. */

#if defined(_OS_WINDOWS)
	#if !defined(_W64)
	#define _W64	/* this is defined for Windows 32/64 bit compatible, otherwise MS C/C++ compiler complains... */
	#endif
#endif

#if defined(_64BPLATFORM)
	typedef _U64 PTR_SIZE;
#else
	#if defined(_OS_WINDOWS)
		typedef _W64 U32 PTR_SIZE;
	#else
		typedef U32 PTR_SIZE;
	#endif
#endif

typedef U8	ByTE;
typedef U16	HALF;
/* please avoid using WORD since it is conflict with Windows definition */
//typedef U32	WORD;
typedef int	BoOL;

#if !defined(TRUE)
	#define TRUE	1
#endif

#if !defined(FALSE)
	#define FALSE	0
#endif

/** LBA is defined for array level LBA, which should be 64 bit */
typedef U64 LBA;

/** I am struggling if I should define the disk LBA as 32 bit or 64 bit. 32 bit 
    disk LBA is as large as 2TB disk capability which should be enough for several
	years in the future. But ATA spec already defined 48 bit LBA for disk. 
	BIOS can benefit from 32 bit disk LBA since the 16bit compiler doesn't support
	64 bit operation ...
 */
typedef U32 DISKLBA;

/* DIMMADDR is actually an offset into DIMM */
typedef U32 DIMMADDR;

#if defined(_X8616B)
	/* 32 bit physical address should be appropriate for BIOS/DOS APP */
	typedef U32 PHYADDR;
#else
	/* for 32/64 bit OS, we will define PHYADDR as 64 bit */
	typedef U64 PHYADDR;
#endif

#if !defined(IN)
#define IN
#endif

#if !defined(OUT)
#define OUT
#endif

#if !defined(OPTIONAL)
#define OPTIONAL
#endif


/* End of basic data type definitions */
/********************************************************************************/

/********************************************************************************/
/**
	Hardware independent SG format. PBM/Raid Engine/DM will all operate on this format.
	CAM should translate it into HW compliant format.
 */
struct _SG32;
typedef struct _SG32 SG32, buf_space* bpSG32;

struct _SG64;
typedef struct _SG64 SG64, buf_space* bpSG64;

/* Please note that the 1st byte of _SGxx is always the control flag byte. It is important
   for this so we can know if it is a 32 bit or 64 bit SG format. */
struct _SG32
{
	U8 CtrlFlag;	/* see below SG_CF_xxx */
	U8 Reserved[3];	/* it is extended to be used as high 8 bit of a 24 bit Count */
	U32 Count;
	U32 StartAddress;
};

struct _SG64
{
	U8 CtrlFlag;	/* see below SG_CF_xxx */
	U8 Reserved[3];
	U32 Count;
	U64 StartAddress;
};

#if defined(_X8616B) || defined(SIMULATE_BIOS_ENV) || defined(__CC_ARM) //|| defined(_SHASTAFW) || defined(NAPA_MARVELL)
#define USE_32BIT_SG_DEFAULT
#endif

/** Things becoming complex if we support mixed SG32 and SG64...
	finally I decided BIOS to use SG32 format and driver use SG64 exclusively. */

#if defined(USE_32BIT_SG_DEFAULT)
	/* BIOS/DOSAPP will use 32 bit SG format */
	typedef SG32 SG;
	#define SG_CF_DEFAULT 0
	#define SG_ASSIGN_STARTADDRESS(StartAddress,Addr) ((StartAddress) = (U32)(Addr))
	#define SG_STARTADDRESS(StartAddress)	(StartAddress)
#else /* !_X8616B */
	/* otherwise we will use 64 bit SG format for convinience */
	typedef SG64 SG;
	#define SG_CF_DEFAULT SG_CF_64B
	#define SG_STARTADDRESS(StartAddress)	((StartAddress).Value)
	#define SG_ASSIGN_STARTADDRESS(StartAddress,Addr) ((StartAddress).Value = (_U64)(Addr))
#endif /* _X8616B */

typedef SG buf_space *bpSG, str_space *spSG;

/* SG table definition */
struct _SGTABLE;
typedef struct _SGTABLE SGTABLE, buf_space* bpSGTABLE;

#if !defined(MAX_SG_ITEM_PER_TABLE)
/* MAX_SG_ITEM_PER_TABLE can be override in cfg_xxx.h */
#if defined(_X8616B) || defined(SIMULATE_BIOS_ENV)
#define MAX_SG_ITEM_PER_TABLE	MAX_DISK_SUPPORT	/* can be even smaller? */
#else
#define MAX_SG_ITEM_PER_TABLE	32
#endif

#endif	/*MAX_SG_ITEM_PER_TABLE*/

struct _SGTABLE
{
	/* actual SG item count, of course, you can get the count 
	   by enumerating the table, but it's convinient to have a count here... */
	U16 SGCount;

	/* you can define your own SG table to save memory, just set 
	   MaxSGCount correctly. */
	U16 MaxSGCount; 

	/* indicates size in byte */
	U32 SizeInByte;

	SG table[MAX_SG_ITEM_PER_TABLE];
};

/* you can use SGTABLE_SIZE to calculate memory requirement if you want to 
   define a non-standard SG table */
#define SGTABLE_SIZE(itemCount) ((itemCount)*sizeof(SG) + OFFSET_OF(SGTABLE,table))

#define SG_CF_EOT	0x80	/* End of table */
#define SG_CF_64B	0x40	/* 64 bit format item */
#define SG_CF_HOST	0x20	/* address in the SG is in host address space */
#define SG_CF_RAW	0x10	/* The address in the SG is un-translated, CAM needs to translate it before passing it to ASIC */
#define SG_CF_DMA	0x08
/*
 * WARNING: Above SG_CF_xxx are virtual flags, not necessarily map to hardware flags. 
 * CAM should NOT use these flags even if the value match with hardware definition.
 * For example, SG_CF_EOT (0x80) is equal to hardware PRD EOT but don't use SG_CF_EOT.
 * SG_CF_xxx can be changed at any time.
 */

#define EndOfSgTable(pSg)		(*((U8*)(pSg))&SG_CF_EOT)
#define MarkEndOfSgTable(pSg)	(*((U8*)(pSg))|=SG_CF_EOT)
#define ClearEndOfSgTable(pSg)	(*((U8*)(pSg))&=(~SG_CF_EOT))
#define SGTableGetSG(pTbl)		((pTbl)->table)
/**
	void SGTableInitialize(bpSGTABLE pSgTbl,U16 maxSgCount);
 */

#define SGTableInitialize(pSgTbl,maxSgCount) do {	\
	(pSgTbl)->SGCount = 0;							\
	(pSgTbl)->MaxSGCount = (maxSgCount);			\
	(pSgTbl)->SizeInByte = 0;						\
} while(0)

/**
	void SGTableStandardInitialize(bpSGTABLE pSgTbl);
 */
#define SGTableStandardInitialize(pSgTbl)	SGTableInitialize(pSgTbl,MAX_SG_ITEM_PER_TABLE)

BoOL SGTableAppendItem(
	bpSGTABLE	pSgTable,
	PTR_SIZE	StartAddress,
	U32			ByteLength,
	U8			CtrlFlags 
	);

#if !defined(_X8616B)
BoOL SGTableAppendItemEx(
	bpSGTABLE	pSgTable,
	U32			StartAddressLo,
	U32			StartAddressHi,
	U32			ByteLength,
	U8			CtrlFlags 
	);
#endif /* _X8616B */

/********************************************************************************/


/********************************************************************************/
/** following functions are defined for compatiblity for compilers that doesn't support
    64 bit operation. We should use this functions instead of direct 64 bit operation even
	if the compiler supports 64 bit operation to keep compatiblity. */
U64 U64_Add(U64 addend1, U64 addend2);
U64 U64_Add_U32(U64 addend1, U32 addend2);
U64 U64_Subtract(U64 Minuend, U64 Subtrahend);
U64 U64_Subtract_U32(U64 Minuend, U32 Subtrahend);
/* isaac add for PCM using */
#if defined(DT64BSUPPORTED)
_U64
u64_divide_u32(_U64 Dividend, U32 Divisor, U32* Remainder);
#endif

U64 U64_Divide(U64 Dividend, U64 Divisor, U64* Remainder);
U64 U64_Divide_U32(U64 Dividend, U32 Divisor, U32* Remainder);
U64 U64_Multiply_U32(U64 Multiplicand, U32 Multiplier);
U32 U64_2_U32(U64 value);
I32 U64_Compare(U64 value1, U64 value2);
U64 U64_Abs_Diff(U64 value1, U64 value2);
#define U64_Zerolize(value) do { value.u.Low = value.u.High = 0; } while(0)
#define U64_Maximize(value)	do { value.u.Low = value.u.High = U32_MAX; } while(0)
#define U64_AssignU32(value, U32Value) do { value.u.Low = (U32Value); value.u.High = 0; } while(0)
/********************************************************************************/


/********************************************************************************/
/** Common component interface definitions */
struct _MEMORY_REQUIREMENT;
typedef struct _MEMORY_REQUIREMENT MEMORY_REQUIREMENT, buf_space *bpMEMORY_REQUIREMENT;

struct _MEMORY_AVAILABLE;
typedef struct _MEMORY_AVAILABLE MEMORY_AVAILABLE, buf_space* bpMEMORY_AVAILABLE;

struct _MEMORY_REQUIREMENT
{
	U32 MinRequirement;
	U32 MaxRequirement;
};

struct _MEMORY_AVAILABLE
{
	bpVOID ComponentExtension;
	bpVOID DMASafeAddress;
	U32 DMASafeSize;
	U32 ComponentExtensionSize;
	PHYADDR DMASafePhysicalAddress;
};

/* return values of xxx_ComponentState */
#define COMP_STATE_UNINITIALIZED	0x00000000
#define COMP_STATE_INITIALIZED		0x00000001
#define COMP_STATE_STARTING			0x00000002
#define COMP_STATE_STOPPED			0x00000004
#define COMP_STATE_STARTED			0x00000005

/* bist flags for xxx_ComponentBist */
#define BIST_CONTROLLER_STOPPED		0x00000000

/** End of common component interface definitions */
/********************************************************************************/


/********************************************************************************/
/* miscellaneous definitions */
#if defined(_64BPLATFORM)
	#define ALSZ	8
#else
	#define ALSZ	4
#endif

#define ALIGN_ROUND(x, align)	( (((x)+(align)-1) / (align)) * (align) )
#define HOST_ALIGN_ROUND(x)		ALIGN_ROUND(x,ALSZ)
#define DIMM_ALIGN_ROUND(x)		ALIGN_ROUND(x,8)	/* always 8 byte alignment for DIMM */
#define ROUNDING(x,align)		(((x)+(align)-1)/(align))

#define OFFSET_OF(type,field)	((U32)(PTR_SIZE)&(((type *)0)->field))
#if defined(_OS_LINUX)
#define FIELD_OFFSET			OFFSET_OF
#endif



#if defined(_X8616B)
#define BASEATTR __based(__segname("_CODE")) 
#else
#define BASEATTR app_space
#endif

/** never force alignment unless absolutely necessary */
#if defined (_COMPILER_GCC_COMPATIBLE)
	#define PACKED __attribute__((packed))
#elif defined(_SHASTAFW)
	//#define PACKED __attribute__((aligned(1)))
	#define PACKED __attribute__((packed))
#else
	#define PACKED
#endif

#define bit(x)				(1 << x)
#define bits(x)				((1 << (x + 1)) - 1)

#define Max(x,y)			(((x) > (y)) ? (x) : (y))
#define Min(x,y)			(((x) < (y)) ? (x) : (y))

#define MAKEU32(hi,lo)		((((U32)(hi))<<16)|(lo))

#define CARRAY_DIM(x)		(sizeof(x)/sizeof((x)[0]))

#define TEST_FLAG(x,flag)	((x)&(flag))

#define IS_EVEN( a )		( ((a)&0x01)==0 )

#define GetPowerOf( a, v )			 \
	do								 \
	{								 \
		U16 temp = ( a );			 \
		( v ) = 0;					 \
		while( ((temp) >>= 1) != 0 ) \
		{							 \
			( v )++;				 \
		}							 \
	} while( 0 )

/*
//
// This structure is used to convert little endian
// ULONGs to SCSI CDB big endians values.
//
*/
typedef union _U64_BYTE {

    struct {
        U8 Byte0;
        U8 Byte1;
        U8 Byte2;
        U8 Byte3;
        U8 Byte4;
        U8 Byte5;
        U8 Byte6;
        U8 Byte7;
    } AsByte;
#if defined(DT64BSUPPORTED)
    _U64 AsULongLong;
#endif /* DT64BSUPPORTED */
} U64_BYTE, *PU64_BYTE;


typedef union _U32_BYTE {

    struct {
        U8 Byte0;
        U8 Byte1;
        U8 Byte2;
        U8 Byte3;
    } AsByte;

    U32 AsULong;
} U32_BYTE, *PU32_BYTE;

typedef union _U16_BYTE {

    struct {
        U8 Byte0;
        U8 Byte1;
    } AsByte;

    U16 AsUShort;
} U16_BYTE, *PU16_BYTE;

/*
//
// Byte reversing macro for converting
// between big- and little-endian formats
//
*/

#define REVERSE_BYTES_U64(Destination, Source) {			\
    PU64_BYTE d = (PU64_BYTE)(Destination);					\
    PU64_BYTE s = (PU64_BYTE)(Source);						\
    d->AsByte.Byte7 = s->AsByte.Byte0;                      \
    d->AsByte.Byte6 = s->AsByte.Byte1;                      \
    d->AsByte.Byte5 = s->AsByte.Byte2;                      \
    d->AsByte.Byte4 = s->AsByte.Byte3;                      \
    d->AsByte.Byte3 = s->AsByte.Byte4;                      \
    d->AsByte.Byte2 = s->AsByte.Byte5;                      \
    d->AsByte.Byte1 = s->AsByte.Byte6;                      \
    d->AsByte.Byte0 = s->AsByte.Byte7;                      \
}


#define REVERSE_BYTES_U32(Destination, Source) {            \
    PU32_BYTE d = (PU32_BYTE)(Destination);					\
    PU32_BYTE s = (PU32_BYTE)(Source);						\
    d->AsByte.Byte3 = s->AsByte.Byte0;                      \
    d->AsByte.Byte2 = s->AsByte.Byte1;                      \
    d->AsByte.Byte1 = s->AsByte.Byte2;                      \
    d->AsByte.Byte0 = s->AsByte.Byte3;                      \
}

#define REVERSE_BYTES_U16(Destination, Source) {            \
    PU16_BYTE d = (PU16_BYTE)(Destination);                 \
    PU16_BYTE s = (PU16_BYTE)(Source);                      \
    d->AsByte.Byte1 = s->AsByte.Byte0;                      \
    d->AsByte.Byte0 = s->AsByte.Byte1;                      \
}

/*
//
// Byte reversing macro for converting
// USHORTS from big to little endian in place
//
*/
#define REVERSE_U16(Short) {            \
    U8 tmp;								\
    PU16_BYTE w = (PU16_BYTE)(Short);   \
    tmp = w->AsByte.Byte0;              \
    w->AsByte.Byte0 = w->AsByte.Byte1;  \
    w->AsByte.Byte1 = tmp;              \
    }
/*
//
// Byte reversing macro for convering
// ULONGS between big & little endian in place
//
*/
#define REVERSE_U32(Long) {				\
    U8 tmp;								\
    PU32_BYTE l = (PU32_BYTE)(Long);	\
    tmp = l->AsByte.Byte3;              \
    l->AsByte.Byte3 = l->AsByte.Byte0;  \
    l->AsByte.Byte0 = tmp;              \
    tmp = l->AsByte.Byte2;              \
    l->AsByte.Byte2 = l->AsByte.Byte1;  \
    l->AsByte.Byte1 = tmp;              \
    }


#define ASSIGN_U32_UNALIGNED(p, v)	do {	\
	bpBUF psrc = (bpBUF)(v);				\
	bpBUF pdst = (bpBUF)(p);				\
	pdst[0] = psrc[0];						\
	pdst[1] = psrc[1];						\
	pdst[2] = psrc[2];						\
	pdst[3] = psrc[3];						\
} while(0)

#define ASSIGN_U16_UNALIGNED(p, v)	do {	\
	bpBUF psrc = (bpBUF)(v);				\
	bpBUF pdst = (bpBUF)(p);				\
	pdst[0] = psrc[0];						\
	pdst[1] = psrc[1];						\
} while(0)

#ifndef _X8616B
extern const U64 BASEATTR U64_ZERO;
#endif

#define U8_MAX		0xff
#define U16_MAX		0xffff
#define U32_MAX		0xffffffffL	/* L postfix is necessary for 16 bit compiler */
#define I32_MAX		0x7fffffffL

/* End of miscellaneous definitions */
/********************************************************************************/

/*
 * Export support for BIOS
 */
#define GEN_EXPORT		GEN_FAR

#if defined(USE_EXPORT)
#define GEN_CALLBACK	GEN_EXPORT
#else
#define GEN_CALLBACK
#endif

#define GEN_PCALLBACK	void GEN_EXPORT*

#if defined(USE_EXPORT)
#define EXP_API(funcName) EXP_##funcName
#else
#define EXP_API(funcName) funcName
#endif

#define DECLARE_EXP(funcName) GEN_EXPORT EXP_API(funcName)

#if defined(_X8616B)
#define PSEUDOBASE // __based(__segname("_CODE")) 
#else	/* _X8616B */
#define PSEUDOBASE app_space
#endif	/* _X8616B */

#define STATIC

#if defined(_X8616B)
/* macros to break C "far" pointers into their segment and offset components
 */

#define GEN_FP_SEG(fp) (*((U16 GEN_FAR *)&(fp)+1))
#define GEN_FP_OFF(fp) (*((U16 GEN_FAR *)&(fp)))

/* macro to construct a far pointer from segment and offset values
 */

#define GEN_MK_FP(seg, offset) (void GEN_FAR *)(((U32)seg << 16) \
    + (U32)(unsigned)offset)

#endif /* _X8616B */


#if defined(_X8616B) && defined(USE_EXPORT)

#define GEN_MK_CALLBACKF( pCallback, funcName ) {	\
	U16 off, curCS;					\
	U16 *p = (U16*) &pCallback;		\
	{								\
	__asm mov off , OFFSET funcName	\
	__asm mov curCS , cs			\
	}								\
	p[0] = off;						\
	p[1] = curCS;					\
}

#else

#define GEN_MK_CALLBACKF(pCallback, funcName)	pCallback = funcName

#endif

#if !defined(EXTERNC)
	#if defined(__cplusplus)
		#define EXTERNC extern "C"
	#else	/* __cplusplus */
		#define EXTERNC
	#endif
#endif	/* EXTERNC */

#if defined(_SHASTAFW)
#define VOLATILE volatile
#else
#define VOLATILE 
#endif

#if defined(_SHASTAFW)
typedef U64 LARGE_INTEGER, *PLARGE_INTEGER;
#endif

#endif /* GEN_DATA_TYPES_H */
