#ifndef NAPA_CONFIG_H
#define NAPA_CONFIG_H

#if defined(_WINAPP)
	#include "cfg_wapp.h"
#elif defined(_WINDRIVER)
	#include "cfg_wdrv.h"
#elif defined(_LINUXDRIVER)
	#include "cfg_linux.h"
#elif defined(_UWDRIVER)
	#include "cfg_uw.h"
#elif defined(BIOS)
	#include "cfg_bios.h"
#elif defined(_SHASTAFW)
	#include "cfg_shasta.h"
#elif defined(__CC_ARM)
	#include "cfg_arm.h"
#else
	#error "What are you going to build?"
#endif

#if !defined(MAX_PORT_SUPPORT)
#define MAX_PORT_SUPPORT	4
#endif

/** Napa abstract channel conception, now channel is the same as disk. low layer CAM
    will help to hide channel from upper layers */
#if !defined(MAX_DISK_SUPPORTED)
	#if defined(CAM_SUPPORT_MASTERSLAVE)
		#define MAX_DISK_SUPPORTED		(MAX_PORT_SUPPORT*2)
	#else	/*CAM_SUPPORT_MASTERSLAVE*/
		#define MAX_DISK_SUPPORTED		MAX_PORT_SUPPORT
	#endif	/*CAM_SUPPORT_MASTERSLAVE*/
#endif /*MAX_DISK_SUPPORTED*/

#if !defined(_X8616B)
	#define MAX_CONCURRENT_IO	32
#else
	#define MAX_CONCURRENT_IO	1
#endif

#define MIN_CONCURRENT_IO		1

#if defined(JBOD_MODE_ONLY)
	#undef ENABLE_DM
	#define NO_RAID5
	#define NO_BGA
	#define DISABLE_ERROR_SIMULATION
	#define DISABLE_DEBUGGING_INTERFACE
	#undef ENABLE_TRANSACTION_LOG
	#undef ENABLE_NON_512_BYTE_SECTOR
	#define DISABLE_SMART_CHECKING
	#undef MULTIPLE_BLOCK_PER_DISK
	#define MAX_BLOCK_PER_DISK	1
#endif

#if !defined(ENABLE_DM) || defined(USE_DMDIMM_ONLY)
/* media patrol must be disabled when BSL is not enabled. */
#undef XH_PATROL
#endif

#if defined(ENABLE_NON_512_BYTE_SECTOR)
/*
 * NON_512_BYTE_SECTOR_SIZE must be defined when ENABLE_NON_512_BYTE_SECTOR is
 * defined and it must be larger than 512 (of course).
 */
#define NON_512_BYTE_SECTOR_SIZE	0x800
#undef PBM_DO_HDMA
#define JBOD_MODE_ONLY
#endif

#define UNREF_PARAM(x) x

#if defined(_X8616B) || defined(SIMULATE_BIOS_ENV)
	#if defined(ENGINE_TIMER_SUPPORT)
		#undef ENGINE_TIMER_SUPPORT
	#endif
#else
	#define ENGINE_TIMER_SUPPORT
#endif

#if (MAX_DISK_SUPPORTED > 4) && !defined(_X8616B)
#define ENGINE_SUPPORT_RMW
#endif

/* BIOS doesn't support error event log now. */
#if !defined(_X8616B) && !defined(BIOS)
#define ENGINE_LOG_ERROR_EVENT
#define LOG_ERR_EVENT_IN_NVRAM
#endif

#if defined(NVRAM_NOT_AVAILABLE)
#undef ENABLE_TRANSACTION_LOG
#undef LOG_ERR_EVENT_IN_NVRAM
#endif

#if defined(XOR_HOST_MEM_CAPABLE) && defined(DMA_HOST_TO_DISK_SUPPORTED)
	#define CC_BYPASS_SUPPORT
	#if defined(_X8616B)
		/* dont undefine PBM_DO_HDMA to save memory during Windows hibernation */
		#undef PBM_DO_HDMA
	#endif	/*_X8616B*/
#endif	/*defined(XOR_HOST_MEM_CAPABLE) && defined(DMA_HOST_TO_DISK_SUPPORTED)*/

#if defined(_WINDRIVER) && (MAX_DISK_SUPPORTED > 4)
/* Component extension size is growing bigger when supported disk is becoming bigger... */
#define USE_STATIC_DATA_FOR_COMPONENT_EXTENSIONS
#endif

#define ENGINE_SEQUENTIAL_SUPPORT

#include "napa_oem.h"

#endif /* NAPA_CONFIG_H */
