#if !defined(OSD_MAIN_H)
#define OSD_MAIN_H

/********************************************************
 *					Data Structure						*
 ********************************************************/
typedef struct _SRB_EXTENSION
{
	U32	SrbExtFlags;
} SRB_EXTENSION, *PSRB_EXTENSION;

/* Component Type Definition. Don't change the order. */
typedef struct _COMPONENT
{
	/* ... Here is the Component Header... */
	U16		ComponentType;		/* see COMP_xxx definition */
	U16		HeaderSize;			/* size of the header only */
	U32		ExtensionSize;		/* size of the ComponentExtension only */
	LPVOID	OSDExtension;		/* fast back reference pointer */

	/* ... Here is the Component Extension... */
	U8		ComponentExtension[1];
} COMPONENT, FAR* PCOMPONENT;

typedef struct _COMP_RESOURCE
{
	LPVOID		ComponentExtension;			/* point to the Component Extension */

	U32		ComponentExtensionSize;		/* size of the Component Extension */
	U32		DMASafeSize;				/* size of the assigned DMA-safe mem */
	LPVOID		DMASafeAddress;				/* virtual address of the assigned DMA-safe Memory */
	PHYADDR		DMASafePhysicalAddress;		/* Physical Address the DMA Safe Memory*/
}COMP_RESOURCE, *PCOMP_RESOURCE;

#define BASE_ADDRESS_MAX_NUM	6

#define HOST_COMP_TYPE_MAXIMUM	2

#define COMP_TYPE_OSD		0
#define COMP_TYPE_MU		1

typedef struct SGTABLE SGTABLE_T;

/* I defined this struture in linux driver, just for using MU component code without modificatioin
    Because MU code use SRB struture whiched originally defined in Windows DDK */
typedef struct _SCSI_REQUEST_BLOCK{
	U16 Length;	
	U8 SrbStatus;
	U8 ScsiStatus;
	U8 PathId;
	U8 TargetId;
	U8 Lun;
	U8 SenseInfoBufferLength;
	void * SenseInfoBuffer;
	U32 DataTransferLength;
	void	*DataBuffer;
	U8 Cdb[16];
} SCSI_REQUEST_BLOCK, *PSCSI_REQUEST_BLOCK;

#define ST_REQ_IN_USE				1
#define PASSTHRU_REQ_HEAD_TYPE 	0x00000001
#define PENDING_REQ_HEAD_TYPE 	0x80000000
#define SLEEPING_REQ_HEAD_TYPE  	0x40000000
#define REQ_HEAD_TYPE_FINISH_WAITING  	0x20000000
typedef struct	_REQ_HEAD {
   SCSI_REQUEST_BLOCK Srb;
   void			*cmd;
   void			(*fn)(void *);   
   unsigned long 	ReqType;
   void  		*pSptd; 
   wait_queue_head_t	waitq;
   struct timer_list req_timer;
   void 		*pExt;
   struct _REQ_HEAD	*next;
} REQ_HEAD, *PREQ_HEAD; 

typedef struct	_DATA_POOL {
   unsigned char *	VBuffer;
   unsigned char *	VDataBuffer;
   u64	PhDataBuffer;
   struct _DATA_POOL	*next;
} DATA_POOL, *PDATA_POOL; 

typedef struct _HOST_CONTROLLER_INFO
{
	U32 SystemIoBusNumber;
	U32 SlotNumber;
	U32 InterruptLevel;
	U32 InterruptVector;
	U32 DeviceID;
	U32 SubDeviceID;
} HOST_CONTROLLER_INFO, * PHOST_CONTROLLER_INFO;

typedef struct _OSD_COMP_EXTENSION 
{ 
	/* Component Management Related Variables */
	COMP_RESOURCE ComponentResource[HOST_COMP_TYPE_MAXIMUM];

	/* Device Extension */
	bpVOID	DeviceExtension;

	U32 State;
	/* Controller Related Variabled */
	HOST_CONTROLLER_INFO ControllerInfo;

	/* Resource Address */
	LPVOID	BaseAddress[BASE_ADDRESS_MAX_NUM];
	dma_addr_t dma_handle;
	LPVOID	DMASafeAddress;			/* The DMA Safe Memory got from OS. */
	ULONG	DMASafeSize; 			/* The DMA Safe Memory Size. */
	PHYADDR	DMASafePhysicalAddress; /* The DMA Safe Memory Physical Address. */

	      	/* OS Related Variables */
      	int			 AdapterIndex;
  	struct pci_dev          *pdev;
  	struct Scsi_Host        *host;             /* pointer to scsi host */
  	int                      	host_no;          /* SCSI host number */
  	unsigned int     	major;            /* control device's major */
  	/* int                   counter;          counter of using control device*/
	
	/* Used for ioctl, no use now 
	unsigned char DataBuffer[MAX_DATA_BUFFER_SIZE*MAX_DATA_BUFFER_COUNT]; */
	
	DATA_POOL DataPool[MAX_DATA_BUFFER_COUNT];
	PDATA_POOL	pDataPoolNext;

	/* OSD Req Head Queue*/
	REQ_HEAD  ReqHeadPool[CAN_QUEUE];
	PREQ_HEAD	pReqCurrent;
	PREQ_HEAD	pReqWaiting;
	
	spinlock_t              datapool_lock; 
	spinlock_t		 timer_lock;

 	struct fasync_struct * async_queue;

	#ifdef SUPPORT_IMAGE_FLASH
	#ifndef DMA_FLASH_IMAGE
	/* One internal SRB. Currently it's used for image flash. */
	SCSI_REQUEST_BLOCK InternalSrb;
	PTR_SIZE FlashSetupFrame;
	#endif
	PSCSI_REQUEST_BLOCK OriginalSrb;
	#endif
	
	struct timer_list InitRAID_Timer;
	
	unsigned char SenseBuffer[SCSI_SENSE_BUFFERSIZE];
	
 	unsigned char		irq;
  	unsigned char		bus_num;
  	unsigned char		devfn;
  	unsigned char		present;
  	unsigned char		bitmap_ioremap;
	unsigned char	CompStartStatus;		/* All components have started or not? */

	unsigned char TimerPoolBuffer[1];	
}OSD_COMP_EXTENSION, buf_space *bpOSD_COMP_EXTENSION;

/********************************************************
 *					macro definition						*
 ********************************************************/

int wrap_print(const char *fmt,...);
 
extern BoOL gIsHibernation;
#define Host_OSD_IsLdrDump(X) gIsHibernation

/* DRIVER_LOCK/UNLOCK pair used for synchronization in interrupt handler or timer handler */
/* In 2.4 kernel, we should use block layer lock: io_request_lock for io completions, but */ 					/* in 2.6 kernel we should take host_lock in Scsi_host struture	instead			  */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0) 

#define DRIVER_LOCK(p,flags)		spin_lock_irqsave(&p->timer_lock, flags)
#define DRIVER_UNLOCK(p,flags)	spin_unlock_irqrestore(&p->timer_lock, flags)

#else

#define DRIVER_LOCK(p,flags)		spin_lock_irqsave(&p->timer_lock, flags)
#define DRIVER_UNLOCK(p,flags)         spin_unlock_irqrestore(&p->timer_lock, flags)

#endif

#define RES_LOCK_INIT		spin_lock_init(&p->res_lock);
#define RES_LOCK		spin_lock_irqsave(&p->res_lock, p->resflags);
#define RES_UNLOCK		spin_unlock_irqrestore(&p->res_lock, p->resflags);


#define VIRT_TO_BUS(a) (u32)virt_to_bus((void *)(a))
#define BUS_TO_VIRT(a) bus_to_virt((unsigned long)(a))

/********************************************************
 *					function protype						*
 ********************************************************/

#if defined(__KERNEL__)

#define PULONG	unsigned int *
#define ULONG unsigned int
#define PUCHAR unsigned char *
#define PUSHORT unsigned short *
#define USHORT unsigned short 
#define UCHAR unsigned char


#endif

void init_data_pool(bpVOID OSDExtension,bpMEMORY_AVAILABLE pMemAvail);
void init_req_pool(bpOSD_COMP_EXTENSION p);

/* SCSI pass through simulation use */
#define SPT_SENSE_LENGTH 		32

typedef struct tagIOCTL_DRIVERVERSION_FRAME
{
	U32 Major;
	U32 Minor;
	U32 Oem;
	U32 BuildNo;
	U32 Signature[2];
	U8 ConsoleID;
	U8 HostNumber;	/* Linux driver only */
	U8 bReserved[2];
	U32 Reserved[3];
} IOCTL_DRIVERVERSION_FRAME, *PIOCTL_DRIVERVERSION_FRAME;

/********************************************************
 *			SCSI Passthrough API Related				*
 ********************************************************/
/* Controller Page */

typedef struct	_VERSION_INFO
{
	U32 VerMajor;
	U32 VerMinor;
	U32 VerOEM;
	U32 VerBuild;
	U32	wReserved[2];
}
VERSION_INFO, *PVERSION_INFO;

typedef struct _APIFRAME_ADAPTER_INFORMATION {
	/*
	 * Defining address as U32 is not compatible with 64 bit system.
	 * I may add another interface to provide 64-bit addresses for AP.
	 */
	U32 BaseAddress[BASE_ADDRESS_MAX_NUM];
	U32 ExpansionROMAddress;

	VERSION_INFO DriverVersion;
	VERSION_INFO BIOSVersion;

	U32 SystemIoBusNumber;
	U32 SlotNumber;
	U32 InterruptLevel;
	U32 InterruptVector;
	U32 VenDevID;				/* Vendor (105A) and Device ID (xxxx) */
	U32 SubVenDevID;			/* Sub-Vendor (105A) and Sub-Device ID (xxxx) */

	U32 DIMMSize;				/* size in MB */
	U8 DIMMType;				/* see DIMM_TYPE_xxx */
	U8 bReserved[3];
	
	U32 ChannelNumber;
	U32 Reserved;

} APIFRAME_ADAPTER_INFORMATION, *PAPIFRAME_ADAPTER_INFORMATION;

/* For ocmdEnclosure: Enclosure Control */
#define API_ENCLOSURE_SUB_GETENCLOSURE		0x01
#define API_ENCLOSURE_SUB_GETSENSOR			0x02
#define API_ENCLOSURE_SUB_LEDLOCATE			0x03
#define API_ENCLOSURE_SUB_MAPDISKID			0x04

typedef struct _APIFRAME_SENSOR
{
	U8 SensorType;
	U8 Reserved[3];
	U32	SensorValue;
} APIFRAME_SENSOR, FAR* PAPIFRAME_SENSOR;

typedef struct _APIFRAME_SENSOR_INFORMATION {
	APIFRAME_SENSOR Sensor[CAM_MAX_SENSOR_PER_ENCLOSURE];
} APIFRAME_SENSOR_INFORMATION, *PAPIFRAME_SENSOR_INFORMATION;

/* Others */
typedef struct _APIFRAME_U32 {
	U32 Value;
} APIFRAME_U32, *PAPIFRAME_U32;

/* Array Hotplug */
#define	IOCTL_PROMISE_REMOVE_DRIVE		0x00010001

typedef struct _IOCTL_PROMISE_REMOVE_DRIVE_FRAME {
	U32	Size;
	U8 	TargetId;
	U8	Action;
	U8	Reserved[2];
} IOCTL_PROMISE_REMOVE_DRIVE_FRAME, *PIOCTL_PROMISE_REMOVE_DRIVE_FRAME;


typedef struct _SRB_IO_CONTROL{
	U32 	HeaderLength;
	U8	Signature[8];
	U32	Timeout;
	U32	ControlCode;
	U32	ReturnCode;
	U32	Length;	
}SRB_IO_CONTROL, *PSRB_IO_CONTROL;

typedef struct _MY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER {
	U8 SrbStatus;
    	U8 ScsiStatus;
    	U8 TargetId;
    	U8 Lun;
    	U8 CdbLength;
    	U8 SenseInfoLength;
    	U32 DataTransferLength;
    	U8 Cdb[16];
    	U8 SenseBuffer[SPT_SENSE_LENGTH];
}MY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER, *PMY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER;

#define MAX_COPY_SIZE 4096
#define MAX_DATA_BUFFER_SIZE 4096 - sizeof(SRB_IO_CONTROL) - sizeof(MY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER)

struct PASS_THRU_IOBUF{
	SRB_IO_CONTROL SrbCtrl;
	MY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER MySptd;
	U8 DataBuffer[MAX_DATA_BUFFER_SIZE];
}; 

/* IOCTL Code */
#define IOCTL_FASTTRAK_GETVERSION           0x00000001
/* Linux SCSI Passthru command wrap use, old IOCTL scheme use only */
#define IOCTL_FASTTRAK_PASSTHRU         _IOWR(0, 1, struct PASS_THRU_IOBUF)  

void
OSD_DumpSrb(
   IN PSCSI_REQUEST_BLOCK Srb
);	
void 
OSD_PrepareSG(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN PSCSI_REQUEST_BLOCK Srb,
	IN bpSGTABLE p_sgTable
	);
void OSD_NotifyBusChange(
	IN PVOID ComponentExtension
);
void OSD_NotifyEvent(
	IN PVOID ComponentExtension
);
void OSD_GetCurrentTime(PU64 time);
void OSD_TrueRAIDReturn(
	IN PVOID OSDExt,
	IN PSCSI_REQUEST_BLOCK Srb
	);
#endif /* OSDI_H */

