#ifndef OSD_MESSAGE_UNIT_H
#define OSD_MESSAGE_UNIT_H

/* How many request MU can handle. */
#define MU_MAX_REQUEST					32
#define MU_MIN_REQUEST					1

/* Because we use head, tail pointer to distinguish payload validity.
 * We'll waste one payload. 
 * In order to handle MU_MAX_REQUEST, we have to have MU_MAX_REQUEST+1 payload. */
#define MU_MAX_PAYLOAD_COUNT			((MU_MAX_REQUEST)+1)
#define MU_MIN_PAYLOAD_COUNT			((MU_MIN_REQUEST)+1)

#define REQUEST_PAYLOAD_HEAD_LENGTH	(OFFSET_OF(MU_REQUEST_PAYLOAD, VariableData))
#define STATUS_PAYLOAD_HEAD_LENGTH	(OFFSET_OF(MU_STATUS_PAYLOAD, VariableData))

#define MU_STATE_STARTING					1
#define MU_STATE_FMU_READY_FOR_HANDSHAKE	2
#define MU_STATE_SEND_HANDSHAKE_FRAME		3
#define MU_STATE_STARTED					4

#define MU_TIMER_ONE_SECOND				1000000

#define MU_INBOUND_MESSAGE_REG0			0x10
#define MU_INBOUND_MESSAGE_REG1			0x14
#define MU_OUTBOUND_MESSAGE_REG0		0x18
#define MU_OUTBOUND_MESSAGE_REG1		0x1c
#define MU_INBOUND_DOORBELL				0x20
#define MU_INBOUND_INTERRUPT_STATUS		0x24
#define MU_INBOUND_INTERRUPT_MASK		0x28
#define MU_OUTBOUND_DOORBELL			0x2c
#define MU_OUTBOUND_INTERRUPT_STATUS	0x30
#define MU_OUTBOUND_INTERRUPT_MASK		0x3c

#define MU_INBOUND_QUEUE_PORT			0x40
#define MU_OUTBOUND_QUEUE_PORT			0x44

/* Definition of the Outbound Interrupt Status Register */
#define MU_OISR_MESSAGE0INTERRUPT		bit(0)
#define MU_OISR_MESSAGED1INTERRUPT		bit(1)
#define	MU_OISR_DOORBELLINTERRUPT		bit(2)
#define MU_OISR_POSTQUEUEINTERRUPT		bit(3)
#define MU_OISR_PCIINTERRUPT			bit(4)

typedef struct _MU_COMP_EXTENSION
{
	bpOSD_COMP_EXTENSION pOSDExtension;
	U32		ComponentState;			/* Maintain the MU State Machine. see MU_STATE_xxx */

	/* Base Address of the ATU */
	PVOID	BaseAddress;			/* To access the firmware register. */
//#ifndef CONSTRAIN_SDRAM_BASE_ADDRESS
	PVOID	SDRAMBase;				/* To access the SDRAM memory. */
//#endif

	/* The Request Payload, Status Payload Window */
	LPVOID	RequestBufferAddr;		/* The beginning address of the Message Queue Window */
	PHYADDR	RequestBufferPhyAdd;

	LPVOID	StatusBufferAddr;		/* The beginning address of the Status Queue Window */
	PHYADDR StatusBufferPhyAddr;
	U16		RequestPayloadCount;
	U16		StatusPayloadCount;

	/* Request and Status Payload Status */
	U32		CmdTag;
	PVOID	OutstandingRequest[MU_MAX_REQUEST];	/* Outstanding Requests which have been submitted to the FMU. */

	U32		RequestPayloadHead;
	U32		RequestPayloadTail;
	U32		StatusPayloadHead;
	U32		StatusPayloadTail;

	U16		OutstandingRequestCount;	/* How many requests got from OS. */
	U16		Reserved;

} MU_COMP_EXTENSION, buf_space * bpMU_COMP_EXTENSION;

typedef struct _MU_INITIALIZE_CONTEXT
{
	PVOID	BaseAddress[BASE_ADDRESS_MAX_NUM];
} MU_INITIALIZE_CONTEXT, buf_space * bpMU_INITIALIZE_CONTEXT;

void 
MU_ComponentGetExtensionSize(
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

void
MU_ComponentGetDMASafeMemoryRequirement(
	IN U32 DeviceID,
	IN U32 SubDeviceID,
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

void
MU_ComponentInitialize(
	IN bpVOID MUExtension,
	IN bpMEMORY_AVAILABLE pMemAvail,
	IN U32 DeviceID,
	IN U32 SubDeviceID,
	IN bpVOID ComponentSpecificContext
	);

void
MU_ComponentStop(
	IN bpVOID MUExtension
	);

void
MU_ComponentStart(
	IN bpVOID ComponentExtension
	);

void 
MU_ComponentRestart(
	IN bpVOID ComponentExtension
	);

/* Handle Request */
NEEDCOMPLETE
MU_HandleRequest(
	IN PVOID MUExt,
	IN PSCSI_REQUEST_BLOCK Srb
	);

BoOL
MU_SendRequest(
	IN PVOID MUExt,
	IN PSCSI_REQUEST_BLOCK Srb
	);

/* Handle Interrupt */
U8 MU_CheckInterrupt(
	IN bpVOID MUExt
	);

void MU_HandleInterrupt(
	IN bpVOID MUExt,
	IN U32 Doorbell
    );

#ifdef NO_FREE_CIRCULAR
U32
MU_PopInboundFreeQueue(
	bpMU_COMP_EXTENSION pMUExtension,
	U32 Tag
	);
#else
U32
MU_PopInboundFreeQueue(
	bpMU_COMP_EXTENSION pMUExtension
	);
#endif

void
MU_PushInboundPostQueue(
	bpMU_COMP_EXTENSION pMUExtension,
	U32 Address
	);

#endif
